package repository

import (
	"gorm.io/gorm"
	req "sghgogs.com/micro/shopping-service/domain/model/request"
	pb "sghgogs.com/micro/shopping-service/proto"
)

// CreateUser(*pb.CreateUserRequest, map[string]interface{}) error
// GetUserList(*pb.GetUserListRequest) ([]req.User, int64, error)
// GetUser(int64) (req.User, error)
// GetUserSelect(int64, string) (req.User, error)
// UpdateUserRoles(int64, []int64) error // 更新用户角色
// UpdateUserPassword(int64, map[string]interface{}) error
// UpdateUserPhoneNumber(int64, map[string]interface{}) error
// ToggleUser(int64, map[string]interface{}) error
// DeleteUser(int64) error

type IRepository interface {
	InitTable() error

	Login(string) (req.User, error)
	Profile(int64) (req.User, error)

	GetAdminUserList(*pb.GetAdminUserListRequest) ([]*req.User, int64, error)
	GetAllAdminUser() ([]*req.User, error)
	GetAdminUser(int64) (*req.User, error)
	CreateAdminUser([]int64, *req.User) error
	UpdateAdminUser(int64, []int64, map[string]interface{}) error
	ToggleAdminUser(int64, pb.StatusEnum, map[string]interface{}) error
	DeleteAdminUser(int64) error
	IsAdminUserExists(interface{}) (bool, error)

	GetUserList(*pb.GetUserListRequest) ([]req.User, int64, error)
	GetUser(int64) (req.User, error)
	CreateUser(*req.User) error
	UpdateUser(int64, map[string]interface{}) error
	DeleteUser(int64) error
	ToggleUser(int64, pb.StatusEnum, map[string]interface{}) error
	IsUserExists(interface{}) (bool, error)

	CreateUserGroup(*pb.CreateUserGroupRequest) error
	GetUserGroup(*pb.GetUserGroupRequest) (req.UserGroup, error)

	CreateAddress(*req.Address) error
	GetAddressList(*pb.GetAddressListRequest) ([]req.Address, int64, error)
	GetAddress(int64) (req.Address, error)
	UpdateAddress(int64, *req.Address) error
	DeleteAddress(int64) error

	GetCategoryList(*pb.GetCategoryListRequest) ([]req.Category, int64, error)
	GetAllCategories() ([]req.Category, error)
	CreateMultipleCategories([]req.Category) error // 批量创建
	DeleteCategory(int64) error

	GetProductList(*pb.GetProductListRequest) ([]req.Product, int64, error)
	GetProduct(int64) (req.Product, error)
	CreateMultipleProducts([]req.Product) error // 批量创建
	UpdateProduct(*pb.UpdateProductRequest) error
	DeleteProduct(int64) error

	CreateShoppingCart(*req.ShoppingCart) error
	CreateMultipleShoppingCartItem([]*req.ShoppingCartItem) error

	CreateOrder(*req.Order) error
	CreateOrderItem(*req.OrderItem) error

	CreatePayment(*req.Payment) error

	// Login(string) (req.AdminUser, error)
	// GetAdminUserList(*pb.GetAdminUserListRequest) ([]*req.AdminUser, int64, error)
	// GetAdminUser(int64) (*req.AdminUser, error)
	// CreateAdminUser(*req.AdminUser, []int64) error
	// UpdateAdminUser(int64, map[string]interface{}, []int64) error
	// ToggleAdminUser(int64, map[string]interface{}) error
	// DeleteAdminUser(int64) error
	// IsAdminUserExists(interface{}) (bool, error)

	IsRoleExists(interface{}) (bool, error)
	GetRoleList(*pb.GetRoleListRequest) ([]*req.Role, int64, error)
	GetAllRoles() ([]*req.Role, error)
	GetRole(int64) (*req.Role, error)
	CreateRole(*pb.CreateRoleRequest) error
	UpdateRole(*pb.UpdateRoleRequest) error
	DeleteRole(int64) error
	ToggleRole(*pb.ToggleRoleRequest) error
	AllRoles() ([]*req.Role, error)

	GetPermissionList(*pb.GetPermissionListRequest) ([]*req.Permission, int64, error)
	GetPermission(int64) (*req.Permission, error)
	CreateMultiplePermissions([]*req.Permission, []int64) error    // 批量创建
	CreatePermission(*req.Permission, []int64) error               // 创建
	UpdatePermission(int64, map[string]interface{}, []int64) error // 更新
	DeletePermission(int64) error
	TogglePermission(int64, map[string]interface{}) error
}

func NewRepository(db *gorm.DB) IRepository {
	return &Repository{
		db: db,
	}
}

type Repository struct {
	db *gorm.DB
}

func (u *Repository) InitTable() error {
	return u.db.AutoMigrate(
		req.User{},
		req.UserGroup{},
		// req.AdminUser{},
		req.Role{},
		req.Permission{},
		req.UserAuth{},
		req.Location{},
		req.Address{},
		req.Category{},
		req.Order{},
		req.OrderItem{},
		req.Payment{},
		req.Product{},
		req.Image{},
		req.ShoppingCart{},
		req.ShoppingCartItem{},
	)
}
